#ifndef METAINF_H
#define METAINF_H

#include <QtCore>


typedef struct {
                char chunk_id[4];
                quint32 chunk_size;
               } t_wav_chunk_hdr;



typedef struct {
                quint16 format;          // = 1 (PCM/uncompressed)
                quint16 num_channels;    // mono = 1, stereo = 2, etc
                quint32 sample_rate;     // 8000, 44100, 48000 etc
                quint32 byte_rate;       // SampleRate * NumChannels * BitsPerSample/8
                quint16 block_align;     // NumChannels * BitsPerSample / 8
                quint16 bits_per_sample; //8, 16 etc
               } t_wav_chunk_fmt;


class CWavReader: public QObject
{

public:

  t_wav_chunk_fmt wav_chunk_fmt;
  double rms;
  bool get_info (const QString fname);
};



class CMetaInfo: public QObject
{
  Q_OBJECT
    
public:

  bool processed;
  
  double bitrate;
  double sample_rate; 
  
  virtual QString stereo_mode() = 0;
  
  virtual void process (const QString &fname) = 0;
 
  CMetaInfo();
  virtual ~CMetaInfo() {};
};


struct frame
{
  int stereo;
  int jsbound;
  int single;
  int II_sblimit;
  int down_sample_sblimit;
  int lsf;
  int mpeg25;
  int down_sample;
  int header_change;
  int lay;
  int error_protection;
  int bitrate_index;
  int sampling_frequency;
  int padding;
  int extension;
  int mode;
  int mode_ext;
  int copyright;
  int original;
  int emphasis;
  int framesize;		/* computed framesize */
};


class CMediaFields: public QObject
{
  Q_OBJECT
    
public:

  bool processed;
  QHash <QString, QString> fields;
   
  virtual void process (const QString &fname) = 0;

  QString charset;

  
  CMediaFields() {processed = false; charset = "Latin1";};
  virtual ~CMediaFields() {};

};



class Cid3v1: public CMediaFields
{
  Q_OBJECT
    
public:

  char chr_signature[3]; 
  char chr_title[31];
  char chr_artist[31];
  char chr_album[31];
  char chr_year[5];
  char chr_comment[31];
  char chr_genre;

  QHash <char, QString> hash;
  
  Cid3v1();
    
  void process (const QString &fname);
};



class CMP3MetaInfo: public CMetaInfo
{
  Q_OBJECT
    
public:

  struct frame fr;
 
  void process (const QString &fname);
  QString stereo_mode();
};


//ID3 tag version 2.4.0
class Cid3v2: public CMediaFields
{
  Q_OBJECT
    
public:

  qint8 ver_major;
  qint8 ver_minor;

 
  void process (const QString &fname);
  
};


#endif // METAINF_H
